import os
from tqdm import tqdm
import numpy as np
import matplotlib.pyplot as plt
import sys

sys.path.insert(0, '../')

from utils import isARawImage, Color, getColorChannel, mergeSingleColorChannelImagesAccordingToBayerFilter, iterativeMean

FOLDERS = {
    'Rafael 23/04/24': '../rafael/230424',
    'RAISE flat-field': '../flat-field/nef',
    'RAISE not flat-field': '../not_flat-field/nef',
}

def getImageMergedColorChannels(filePath):
    imageColorChannels = {color: getColorChannel(filePath, color) for color in Color}
    imageMergedColorChannels = mergeSingleColorChannelImagesAccordingToBayerFilter(imageColorChannels)
    return imageMergedColorChannels

def getEstimatedPrnu(folder):
    meanImages = iterativeMean()
    filePaths = [f'{folder}/{file}' for file in os.listdir(folder) if isARawImage(file)]

    for filePath in tqdm(filePaths, 'Compute mean image'):
        imageMergedColorChannels = getImageMergedColorChannels(filePath)
        meanImages.add(imageMergedColorChannels)

    # Is not there anything clever than this to do? See [Robust_image_source_identification_on_modern_smartphones/issues/72](https://gitea.lemnoslife.com/Benjamin_Loison/Robust_image_source_identification_on_modern_smartphones/issues/72).
    estimatedPrnu = iterativeMean()
    for filePath in tqdm(filePaths, 'Compute estimated PRNU'):
        imageMergedColorChannels = getImageMergedColorChannels(filePath)
        estimatedPrnu.add(imageMergedColorChannels - meanImages.mean)
    return estimatedPrnu.mean

estimatedPrnu = getEstimatedPrnu(FOLDERS['Rafael 23/04/24'])

plt.imshow(estimatedPrnu)
plt.show()